﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Security;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;

namespace WLV_LoadItemsAsync
{
    public partial class Form1 : Form
    {
        private int activatedItems = 0;
        CustomCreationFilter filter = null;
        string lastFolderPath = string.Empty;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var colFileSize = this.ultraListView1.SubItemColumns.Add("FileSize");
            var colFileType = this.ultraListView1.SubItemColumns.Add("FileType");
            var colDateModified = this.ultraListView1.SubItemColumns.Add("DateModified");

            colFileSize.DataType = typeof(int);
            colFileSize.Format = "#,###,##0 KB";
            colFileSize.SubItemAppearance.TextHAlign = HAlign.Right;
            colFileType.DataType = typeof(string);
            colDateModified.DataType = typeof(DateTime);
            colFileType.Text = "Type";
            colDateModified.Text = "Date Modified";

            string shortDateFormat =
              System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern;
            string shortTimeFormat =
              System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortTimePattern;
            colDateModified.Format = string.Format("{0} {1}", shortDateFormat, shortTimeFormat);

            this.ultraListView1.MainColumn.DataType = typeof(string);
            this.ultraListView1.MainColumn.Text = "Name";

            this.ultraListView1.View = UltraListViewStyle.Details;

            filter = new CustomCreationFilter();
            ultraListView1.CreationFilter = filter;

            filter.ItemInitialized += filter_ItemInitialized;

            ultraListView1.ViewSettingsDetails.FullRowSelect = true;
        }

        void filter_ItemInitialized(object sender, ItemActivatedEventArgs e)
        {
            ultraLabel3.Text = "Initialized Items: "+ ++activatedItems;
        }

        private void GetDirectoryItems(string folderPath)
        {
            // Regenerate the list view only if the user choses a different folder path
            if (folderPath == this.lastFolderPath)
                return;
            else
                lastFolderPath = folderPath;

            // I added code to dispose of previous resource 
            // If your application can show info about different directories
            if (ultraListView1.Items.Count > 0)
            {
                foreach (var item in ultraListView1.Items)
                {
                    item.Dispose();
                }
                ultraListView1.Items.Clear();
            }

            // Remove previously cached objects in the filter.
            if (filter != null)
                filter.ClearCachedInfo();

            ultraLabel2.Text = "Loaded items: 0";
            ultraLabel3.Text = "Initialized Items: 0";
            activatedItems = 0;

            DirectoryInfo cDriveInfo = new DirectoryInfo(folderPath);

            DirectoryInfo[] directories = cDriveInfo.GetDirectories();
            FileInfo[] files = cDriveInfo.GetFiles();

            UltraListViewItem[] items = new UltraListViewItem[directories.Length + files.Length];

            for (int i = 0; i < directories.Length; i++)
            {
                DirectoryInfo directoryInfo = directories[i];

                UltraListViewItem item = new UltraListViewItem()
                {
                    Key = directoryInfo.FullName,
                    Value = directoryInfo.Name,
                };

                items[i] = item;
            }

            for (int i = 0; i < files.Length; i++)
            {
                FileInfo fileInfo = files[i];

                UltraListViewItem item = new UltraListViewItem()
                {
                    Key = fileInfo.FullName,
                    Value = fileInfo.Name,
                };

                items[directories.Length + i] = item;
            }

            ultraListView1.Items.AddRange(items);

            ultraLabel2.Text = "Loaded items: " + ultraListView1.Items.Count;
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                GetDirectoryItems(folderBrowserDialog1.SelectedPath);
            }
        }

        private void ultraListView1_ItemDoubleClick(object sender, ItemDoubleClickEventArgs e)
        {
            if (Utils.IsDirectory(e.Item.Key))
            {
                GetDirectoryItems(e.Item.Key);
            }

        }

        private void ultraListView1_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Back && lastFolderPath != String.Empty)
            {
                var directoryInfo = new DirectoryInfo(lastFolderPath);

                if (directoryInfo.Parent != null)
                    GetDirectoryItems(directoryInfo.Parent.FullName);
            }
        }
    }
}
